@@ -5,21 +5,33 @@
  * modify it under the terms of the GNU General Public License as            *
  * published by the Free Software Foundation; either version 2 of            *
  * the License, or (at your option) any later version.                       *
- *                                                                           *           
+ *                                                                           *
  * This program is distributed in the hope that it will be useful,           *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
  * GNU General Public License for more details.                              *
- *                                                                           *   
+ *                                                                           *
  * You should have received a copy of the GNU General Public License         *
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
  *****************************************************************************
  */
 
-#include <QProcess>
-#include <QRegExp>
-#include <KStandardDirs>
-#include <KTextEditor/View>
+#include "context.h"
+
+#include "items/keyword.h"
+#include "items/importfile.h"
+#include "items/functiondeclaration.h"
+#include "items/implementfunction.h"
+
+#include "worker.h"
+#include "helpers.h"
+#include "duchain/pythoneditorintegrator.h"
+#include "duchain/expressionvisitor.h"
+#include "duchain/declarationbuilder.h"
+#include "duchain/helpers.h"
+#include "duchain/types/unsuretype.h"
+#include "duchain/navigation/navigationwidget.h"
+#include "parser/astbuilder.h"
 
 #include <language/duchain/functiondeclaration.h>
 #include <language/duchain/classdeclaration.h>
@@ -35,21 +47,10 @@
 #include <interfaces/idocumentcontroller.h>
 #include <project/projectmodel.h>
 
-#include "parser/astbuilder.h"
-#include "duchain/pythoneditorintegrator.h"
-#include "duchain/expressionvisitor.h"
-#include "duchain/declarationbuilder.h"
-#include "duchain/helpers.h"
-#include "duchain/types/unsuretype.h"
-#include "duchain/navigation/navigationwidget.h"
-
-#include "worker.h"
-#include "context.h"
-#include "items/keyword.h"
-#include "items/importfile.h"
-#include "items/functiondeclaration.h"
-#include "items/implementfunction.h"
-#include "helpers.h"
+#include <QProcess>
+#include <QRegExp>
+#include <KStandardDirs>
+#include <KTextEditor/View>
 
 using namespace KTextEditor;
 using namespace KDevelop;
@@ -61,6 +62,24 @@ PythonCodeCompletionContext::ItemTypeHint PythonCodeCompletionContext::itemTypeH
     return m_itemTypeHint;
 }
 
+ExpressionVisitor* visitorForString(QString str, DUContext* context, CursorInRevision scanUntil = CursorInRevision::invalid()) {
+    KDevPG::MemoryPool pool;
+    QSharedPointer<AstBuilder> builder(new AstBuilder(&pool));
+    CodeAst* tmpAst = builder->parse(KUrl(), str);
+    if ( tmpAst ) {
+        ExpressionVisitor* v = new ExpressionVisitor(context);
+        v->m_forceGlobalSearching = true;
+        if ( scanUntil.isValid() ) {
+            v->m_scanUntilCursor = scanUntil;
+            v->m_reportUnknownNames = true;
+        }
+        v->visitCode(tmpAst);
+        return v;
+    }
+    kDebug() << "Completion requested for syntactically invalid expression";
+    return 0;
+}
+
 QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bool& abort, bool fullCompletion)
 {
     QList<CompletionTreeItemPointer> resultingItems;
@@ -73,7 +92,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
     if ( m_operation != FunctionCallCompletion ) {
         KeywordItem::Flags f = (KeywordItem::Flags) ( KeywordItem::ForceLineBeginning | KeywordItem::ImportantItem );
         // TODO group those correctly so they appear at the top
-        if ( m_position.line == 0 && ( m_text.startsWith("#") || m_text.isEmpty() ) ) {
+        if ( m_position.line == 0 && ( m_text.startsWith('#') || m_text.isEmpty() ) ) {
             QString i18ndescr = i18n("insert Shebang line");
             resultingItems << CompletionTreeItemPointer(new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this),
                                                         "#!/usr/bin/env python\n", i18ndescr, f));
@@ -83,7 +102,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
                                                         "#!/usr/bin/env python3\n", i18ndescr, f));
         }
         // TODO fixme
-        else if ( m_position.line <= 1 && m_text.endsWith("#") ) {
+        else if ( m_position.line <= 1 && m_text.endsWith('#') ) {
             resultingItems << CompletionTreeItemPointer(new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this),
                                                         "# -*- Coding:utf-8 -*-\n\n", i18n("specify document encoding"), f));
         }
@@ -98,30 +117,23 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
         kDebug() << "no code completion";
     }
     else if ( m_operation == PythonCodeCompletionContext::GeneratorVariableCompletion ) {
-        QList<KeywordItem*> completionItems;
-        KDevPG::MemoryPool pool;
-        AstBuilder* builder = new AstBuilder(&pool);
-        CodeAst* tmpAst = builder->parse(KUrl(), m_guessTypeOfExpression);
-        if ( tmpAst ) {
-            lock.unlock();
-            ExpressionVisitor* v = new ExpressionVisitor(m_duContext.data());
-            v->m_forceGlobalSearching = true;
-            v->m_scanUntilCursor = m_position;
-            v->m_reportUnknownNames = true;
-            v->visitCode(tmpAst);
-            lock.lock();
-            if ( not v->m_unknownNames.isEmpty() ) {
+        QList<KeywordItem*> items;
+        lock.unlock();
+        ExpressionVisitor* v = visitorForString(m_guessTypeOfExpression, m_duContext.data(), m_position);
+        lock.lock();
+        if ( v ) {
+            if ( ! v->m_unknownNames.isEmpty() ) {
                 if ( v->m_unknownNames.size() >= 2 ) {
                     // we only take the first two, and only two. It gets too much items otherwise.
                     QStringList combinations;
                     combinations << v->m_unknownNames.at(0) + ", " + v->m_unknownNames.at(1);
                     combinations << v->m_unknownNames.at(1) + ", " + v->m_unknownNames.at(0);
                     foreach ( const QString& c, combinations ) {
-                        completionItems << new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this), "" + c + " in ", "");
+                        items << new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this), "" + c + " in ", "");
                     }
                 }
                 foreach ( const QString& n, v->m_unknownNames ) {
-                    completionItems << new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this), "" + n + " in ", "");
+                    items << new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this), "" + n + " in ", "");
                 }
             }
             else {
@@ -129,9 +141,8 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
             }
             delete v;
         }
-        delete builder;
         
-        foreach ( KeywordItem* item, completionItems ) {
+        foreach ( KeywordItem* item, items ) {
             resultingItems << CompletionTreeItemPointer(item);
         }
         
@@ -221,7 +232,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
                 QList<DeclarationDepthPair> declarations = c->allDeclarations(
                     CursorInRevision::invalid(), m_duContext->topContext(), false
                 );
-                foreach ( DeclarationDepthPair d, declarations ) {
+                foreach ( const DeclarationDepthPair& d, declarations ) {
                     if ( FunctionDeclaration* funcDecl = dynamic_cast<FunctionDeclaration*>(d.first) ) {
                         // python does not have overloads or similar, so comparing the function names is enough.
                         const IndexedString identifier = funcDecl->identifier().identifier();
@@ -260,7 +271,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
             QList<DeclarationDepthPair> validDeclarations;
             ClassDeclaration* current = 0;
             StructureType::Ptr type;
-            foreach ( DeclarationDepthPair d, m_duContext->topContext()->allDeclarations(CursorInRevision::invalid(), m_duContext->topContext()) ) {
+            foreach ( const DeclarationDepthPair d, m_duContext->topContext()->allDeclarations(CursorInRevision::invalid(), m_duContext->topContext()) ) {
                 if ( ( current = dynamic_cast<ClassDeclaration*>(d.first) ) ) {
                     if ( current->baseClassesSize() ) {
                         FOREACH_FUNCTION( const BaseClassInstance& base, current->baseClasses ) {
@@ -285,9 +296,25 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
     }
     else if ( m_operation == PythonCodeCompletionContext::InheritanceCompletion ) {
         kDebug() << "InheritanceCompletion";
-        QList<DeclarationDepthPair> declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
+        QList<DeclarationDepthPair> declarations;
+        if ( ! m_guessTypeOfExpression.isEmpty() ) {
+            // The class completion is a member access
+            ExpressionVisitor* v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
+            if ( v ) {
+                TypePtr<StructureType> cls = StructureType::Ptr::dynamicCast(v->lastType());
+                if ( cls && cls->declaration(m_duContext->topContext()) ) {
+                    if ( DUContext* internal = cls->declaration(m_duContext->topContext())->internalContext() ) {
+                        declarations = internal->allDeclarations(m_position, m_duContext->topContext(), false);
+                    }
+                }
+                delete v;
+            }
+        }
+        else {
+            declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
+        }
         QList<DeclarationDepthPair> remainingDeclarations;
-        foreach ( DeclarationDepthPair d, declarations ) {
+        foreach ( const DeclarationDepthPair& d, declarations ) {
             Declaration* r = Helper::resolveAliasDeclaration(d.first);
             if ( r and r->identifier().identifier().str().contains("__kdevpythondocumentation_builtin") ) {
                 continue;
@@ -299,15 +326,8 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
         resultingItems.append(declarationListToItemList(remainingDeclarations));
     }
     else if ( m_operation == PythonCodeCompletionContext::MemberAccessCompletion ) {
-        KDevPG::MemoryPool pool;
-        AstBuilder* builder = new AstBuilder(&pool);
-        CodeAst* tmpAst = builder->parse(KUrl(), m_guessTypeOfExpression);
-        if ( tmpAst ) {
-            lock.unlock();
-            ExpressionVisitor* v = new ExpressionVisitor(m_duContext.data());
-            v->m_forceGlobalSearching = true;
-            v->visitCode(tmpAst);
-            lock.lock();
+        ExpressionVisitor* v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
+        if ( v ) {
             if ( v->lastType() ) {
                 kDebug() << v->lastType()->toString();
                 resultingItems << getCompletionItemsForType(v->lastType());
@@ -320,7 +340,6 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
         else {
             kWarning() << "Completion requested for syntactically invalid expression, not offering anything";
         }
-        delete builder;
     }
     else {
         // it's stupid to display a 3-letter completion item on manually invoked code completion and makes everything look crowded
@@ -332,11 +351,46 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::completionItems(bo
                 resultingItems << CompletionTreeItemPointer(k);
             }
         }
+        if ( m_operation == PythonCodeCompletionContext::NewStatementCompletion ) {
+            // Eventually suggest initializing class members from constructor arguments
+            if ( Declaration* decl = duContext()->owner() ) {
+                if ( DUContext* args = DUChainUtils::getArgumentContext(duContext()->owner()) ) {
+                    if ( decl->isFunctionDeclaration() && decl->identifier() == KDevelop::Identifier("__init__") ) {
+                        // the current context actually belongs to a constructor
+                        foreach ( const Declaration* argument, args->localDeclarations() ) {
+                            const QString argName = argument->identifier().toString();
+                            // Do not suggest "self.self = self"
+                            if ( argName == "self" ) {
+                                continue;
+                            }
+                            bool usedAlready = false;
+                            // Do not suggest arguments which already have a use in the context
+                            // This is uesful because you can then do { Ctrl+Space Enter Enter } while ( 1 )
+                            // to initialize all available class variables, without using arrow keys.
+                            for ( int i = 0; i < duContext()->usesCount(); i++ ) {
+                                if ( duContext()->uses()[i].usedDeclaration(duContext()->topContext()) == argument ) {
+                                    usedAlready = true;
+                                    break;
+                                }
+                            }
+                            if ( usedAlready ) {
+                                continue;
+                            }
+                            const QString value = "self." + argName + " = " + argName;
+                            KeywordItem* item = new KeywordItem(KDevelop::CodeCompletionContext::Ptr(this),
+                                                                value, i18n("Initialize property"),
+                                                                KeywordItem::ImportantItem);
+                            resultingItems.append(CompletionTreeItemPointer(item));
+                        }
+                    }
+                }
+            }
+        }
         if ( abort ) {
             return QList<CompletionTreeItemPointer>();
         }
         QList<DeclarationDepthPair> declarations = m_duContext->allDeclarations(m_position, m_duContext->topContext());
-        foreach ( DeclarationDepthPair d, declarations ) {
+        foreach ( const DeclarationDepthPair& d, declarations ) {
             if ( d.first and d.first->context()->type() == DUContext::Class ) {
                 declarations.removeAll(d);
             }
@@ -358,7 +412,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
     QList<CompletionTreeItemPointer> items;
     
     DeclarationPointer currentDeclaration;
-    DUChainPointer<Declaration> checkDeclaration;
+    Declaration* checkDeclaration = 0;
     int count = declarations.length();
     for ( int i = 0; i < count; i++ ) {
         if ( maxDepth && maxDepth > declarations.at(i).second ) {
@@ -368,13 +422,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
         currentDeclaration = DeclarationPointer(declarations.at(i).first);
         
         PythonDeclarationCompletionItem* item = 0;
-        AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(currentDeclaration.data());
-        if ( alias ) {
-            checkDeclaration = DUChainPointer<Declaration>(alias->aliasedDeclaration().declaration());
-        }
-        else {
-            checkDeclaration = currentDeclaration;
-        }
+        checkDeclaration = Helper::resolveAliasDeclaration(currentDeclaration.data());
         if ( checkDeclaration ) {
             AbstractType::Ptr type = checkDeclaration->abstractType();
             if ( type && ( type->whichType() == AbstractType::TypeFunction || type->whichType() == AbstractType::TypeStructure ) ) {
@@ -383,6 +431,9 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
             else {
                 item = new PythonDeclarationCompletionItem(currentDeclaration, KDevelop::CodeCompletionContext::Ptr(this));
             }
+            if ( ! m_matchAgainst.isEmpty() ) {
+                item->addMatchQuality(identifierMatchQuality(m_matchAgainst, checkDeclaration->identifier().toString()));
+            }
             items << CompletionTreeItemPointer(item);
         }
     }
@@ -443,7 +494,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::getCompletionItems
             // also, discard all magic functions from autocompletion
             // TODO rework this, it's maybe not the most elegant solution possible
             // TODO rework the magic functions thing, I want them sorted at the end of the list but KTE doesn't seem to allow that
-            foreach ( DeclarationDepthPair current, declarations ) {
+            foreach ( const DeclarationDepthPair& current, declarations ) {
                 if ( current.first->context() != builtinTopContext && ! current.first->identifier().identifier().str().startsWith("__") ) {
                     keepDeclarations.append(current);
                 }
@@ -498,7 +549,15 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::findIncludeItems(I
         DUContext* c = internalContextForDeclaration(top, item.remainingIdentifiers);
         kDebug() << "  GOT:" << c;
         if ( c ) {
+            int begin = items.size();
             items << declarationListToItemList(c->localDeclarations().toList());
+            // tell function declaration items not to add brackets
+            // TODO this is a hack. :(
+            for ( int i = begin; i < items.size(); i++ ) {
+                if ( FunctionDeclarationCompletionItem* item = dynamic_cast<FunctionDeclarationCompletionItem*>(items[i].data()) ) {
+                    item->setIsImportItem(true);
+                }
+            }
         }
         else {
             // do better next time
@@ -510,7 +569,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::findIncludeItems(I
         // append all python files in the directory
         foreach ( QFileInfo file, contents ) {
             // TODO windows
-            if ( file.fileName().startsWith(".") ) {
+            if ( file.fileName().startsWith('.') ) {
                 continue;
             }
             kDebug() << " > CONTENT:" << file.absolutePath() << file.fileName();
@@ -592,10 +651,10 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::includeItemsForSub
     // Thus, we first generate a list of possible paths, then match them against those which actually exist
     // and then gather all the items in those paths.
     
-    foreach (KUrl currentPath, searchPaths) {
+    foreach ( KUrl currentPath, searchPaths ) {
         kDebug() << "Searching: " << currentPath << subdirs;
         int identifiersUsed = 0;
-        foreach ( QString subdir, subdirs ) {
+        foreach ( const QString& subdir, subdirs ) {
             currentPath.cd(subdir);
             QFileInfo d(currentPath.path());
             kDebug() << currentPath << d.exists() << d.isDir();
@@ -695,7 +754,6 @@ PythonCodeCompletionContext::PythonCodeCompletionContext(DUContextPointer contex
     ExpressionParser parser(textWithoutStrings);
     TokenList allExpressions = parser.popAll();
     allExpressions.reset(1);
-    allExpressions.length();
     ExpressionParser::Status firstStatus = allExpressions.last().status;
     
     // TODO reuse already calculated information
@@ -760,6 +818,9 @@ PythonCodeCompletionContext::PythonCodeCompletionContext(DUContextPointer contex
             // TODO: optimally, filter out classes we already inherit from. That's a bonus, tough.
             if ( allExpressions.at(1).status == ExpressionParser::ClassFound ) {
                 // show only items of type "class" for completion
+                if ( allExpressions.at(allExpressions.length()-1).status == ExpressionParser::MemberAccessFound ) {
+                    m_guessTypeOfExpression = allExpressions.at(allExpressions.length()-2).expression;
+                }
                 m_operation = InheritanceCompletion;
                 return;
             }
@@ -933,6 +994,11 @@ PythonCodeCompletionContext::PythonCodeCompletionContext(DUContextPointer contex
         }
         return;
     }
+
+    if ( firstStatus == ExpressionParser::EqualsFound && allExpressions.length() >= 2 ) {
+        m_operation = DefaultCompletion;
+        m_matchAgainst = allExpressions.at(allExpressions.length() - 2).expression;
+    }
 }
 
 }
